@@ -115,9 +115,7 @@ int process_inner_request(struct ProfileData *pr, xmlDocPtr inner_req)
         msa_xmlDocDump(stderr, inner_request);
         msa_debug("INNER RESPONSE\n");
         msa_xmlDocDump(stderr, inner_response);
-        /*
-           TODO Check if error code = 6 sleep(1) and try request
-        */
+
         tmp_node =  xpath_get_node(XPATH_TAG_ERROR_CODE, inner_response);
         if (tmp_node != NULL){
             gchar* code = NULL;
@@ -173,6 +171,7 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
     gchar *tmp = NULL;
     gchar *resp_str = NULL;
     gchar* data = NULL;
+    gchar* strBase64 = NULL;
     gchar* http_type = NULL;
     gchar* params = NULL;
     gchar* encoded_params = NULL;
@@ -222,7 +221,8 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
             || g_strcmp0(http_type, "get") ==0 
             || g_strcmp0(http_type, "post") ==0 
             || g_strcmp0(http_type, "download") ==0 
-            || g_strcmp0(http_type, "webauth") == 0) {
+            || g_strcmp0(http_type, "webauth") == 0
+            || g_strcmp0(http_type, "webcaptcha") == 0) {
 
             node = xpath_get_node("//InnerRequest/url", inner_req);
             if (node != NULL) {
@@ -254,11 +254,18 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
 
             if(g_strcmp0(http_type, "webauth") == 0){
                 data = call_webauth(url, pr);
+            } else if(g_strcmp0(http_type, "webcaptcha") == 0){
+                http_code = http_get(url, info_proxy, info_port, &data, &data_size);
+                if(http_code < 300){
+                    strBase64 = g_base64_encode(data, data_size);
+                    g_free(data);
+                    data = call_webcaptcha(strBase64, pr);
+                }
             } else if(g_strcmp0(http_type, "get") == 0 || g_strcmp0(http_type, "download") == 0){
-                http_code = http_get(url, info_proxy, info_port, NULL, &data, &data_size);
+                http_code = http_get(url, info_proxy, info_port, &data, &data_size);
             } else if(g_strcmp0(http_type, "post") == 0){
                 post_data = g_strsplit(url,"?",2);
-                http_code = http_post(post_data[0], info_proxy, info_port, post_data[1], NULL, &data, &data_size);
+                http_code = http_post(post_data[0], info_proxy, info_port, post_data[1], &data, &data_size);
                 g_strfreev(post_data);
             } else if(g_strcmp0(http_type, "upload") == 0){
                 msa_debug("UPLOADING...");
@@ -275,13 +282,13 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
                     file->params = g_strsplit_set(post_data[1], "&=", 100);
                 else
                     file->params = NULL;
-                http_code = http_upload_file(post_data[0], info_proxy, info_port, NULL, &data, pr, file);
+                http_code = http_upload_file(post_data[0], info_proxy, info_port, &data, file);
                 upfile_remove(file);
                 g_strfreev(post_data);
             }
             g_free(url);
 
-            if (http_code >= 400 && http_code < 500){                                        
+            if (http_code >= 400 && http_code < 500){
                 tmp = g_strdup_printf("http code %d", http_code);
                 error_message(DRV_NETW_ERROR_CODE, &err_resp, DRV_NETW_ERROR, tmp);
                 g_free(tmp);
@@ -332,6 +339,11 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
                     g_free(data);
                     data = g_strdup(resp_str);
                     g_free(resp_str);
+                } else if (g_strcmp0(parser_type, PARSER_WEBCAPTCHA) == 0) {
+                    webcaptcha_parser(inner_req, pr, data, 0, &resp_str);
+                    g_free(data);
+                    data = g_strdup(resp_str);
+                    g_free(resp_str);
                 }
 
             }
@@ -581,6 +593,35 @@ gchar* call_webauth(gchar* url, struct ProfileData *cp)
    return back_url;
 }
 
+/**
+ * call webauth application (function webcaptcha)
+ *
+ * @param cp - current profile
+ * @return 0 on success or 1 otherwise.
+ */
+gchar* call_webcaptcha(gchar* strBase64, struct ProfileData *cp)
+{
+    gchar* result = NULL;
+
+    msa_debug("call_webcaptcha: START");
+
+    if (cp == NULL) {
+        msa_debug("can't authorize, no profile!");
+        return NULL;
+    }
+    if (strBase64 == NULL) {
+        msa_debug("can't show captcha, no image!");
+        return NULL;
+    }
+
+    msa_debug("Try call webauth with %s = %s", CLIENT_ID_PARAM, cp->id);
+
+    /* patterns[1] its TITLE_PARAM, patterns[3] its ACCOUNT_ID_PARAM */
+    result = webcaptcha(strBase64, DRIVER_ID);
+
+   return result;
+}
+
 /**
  * Get class of request
  *
@@ -661,6 +702,7 @@ void curl_set_options(CURL *c, const char* proxy, int port)
     curl_easy_setopt(c, CURLOPT_PROXYPORT, curPort);
     curl_easy_setopt(c, CURLOPT_TIMEOUT, 120);
     curl_easy_setopt(c, CURLOPT_FOLLOWLOCATION, 1);
+    curl_easy_setopt(c, CURLOPT_NOPROGRESS, 1);
 }
 
 /**
@@ -669,14 +711,15 @@ void curl_set_options(CURL *c, const char* proxy, int port)
  * @param url - request
  * @param proxy - proxy address 
  * @param port - proxy port
- * @param header - buffer for header of response
  * @param body - buffer for body of response
  * @return http server response code 
  */
-int http_get(const char* url, const char* proxy, int port, char** header, gchar** body, int* data_size)
+int http_get(const char* url, const char* proxy, int port, gchar** body, int* data_size)
 {
     CURL* c;
     long code;
+    long request_size = 0;
+    double size = 0;
     char* type2;
     char* encoded_url = NULL;
     CURLcode errornum;
@@ -716,18 +759,27 @@ int http_get(const char* url, const char* proxy, int port, char** header, gchar*
     }
 
     free(encoded_url);
-    errornum = curl_easy_getinfo(c, CURLINFO_RESPONSE_CODE, &code );
-    msa_debug("CODE %ld", code);
+
+        errornum = curl_easy_getinfo(c, CURLINFO_RESPONSE_CODE, &code );
+        msa_debug("CODE %ld", code);
+
+        errornum = curl_easy_getinfo(c, CURLINFO_SIZE_UPLOAD, &size );
+        msa_debug("UPLOAD %f", size);
+
+        errornum = curl_easy_getinfo(c, CURLINFO_REQUEST_SIZE, &request_size );
+        msa_debug("REQUEST_SIZE %ld", request_size);
+
+	upload_size += size;
+	upload_size += request_size;
 
     /* type2 = g_malloc(sizeof(char)*50); */
     errornum = curl_easy_getinfo(c, CURLINFO_CONTENT_TYPE, &type2 );
     msa_debug("TYPE %s", type2);
 
-    if (header != NULL) {
-        *header = g_strdup(chunk2.memory);
-    }
+	download_size += chunk2.size;
+    download_size += chunk.size;
 
-    msa_debug("HEADER  %s\n", chunk2.memory);
+    msa_debug("HEADER  %s\n header size = %d", chunk2.memory, chunk2.size);
     if(chunk2.memory)
         free(chunk2.memory);
 
@@ -753,15 +805,16 @@ int http_get(const char* url, const char* proxy, int port, char** header, gchar*
  * @param proxy - proxy address
  * @param port - proxy port
  * @param post_data - data to send
- * @param header - buffer for header of response
  * @param body - buffer for body of response
  * @return http server response code
  */
 
-int http_post(const char* url, const char* proxy, int port, char* post_data, char** header, char** body, int* data_size)
+int http_post(const char* url, const char* proxy, int port, char* post_data, char** body, int* data_size)
 {
     CURL* c;
     long code;
+    long request_size = 0;
+    double size = 0;
     CURLcode errornum;
     struct MemoryStruct chunk,chunk2;
 
@@ -771,7 +824,6 @@ int http_post(const char* url, const char* proxy, int port, char* post_data, cha
     chunk2.memory = NULL;
     chunk2.size = 0;
 
-
     c = curl_easy_init();
     msa_debug("%s:%d:%s:url in http_post %s, proxy=%s:%d", __FILE__, __LINE__, __FUNCTION__, url, proxy, port);
 
@@ -782,9 +834,6 @@ int http_post(const char* url, const char* proxy, int port, char* post_data, cha
     curl_easy_setopt(c, CURLOPT_POST, 1);
     curl_easy_setopt(c, CURLOPT_POSTFIELDS, post_data);
     curl_easy_setopt(c, CURLOPT_POSTFIELDSIZE, strlen(post_data));
-    curl_easy_setopt(c, CURLOPT_FOLLOWLOCATION, 1);
-
-    curl_easy_setopt(c, CURLOPT_COOKIEFILE, "");
 
     curl_easy_setopt(c, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
     curl_easy_setopt(c, CURLOPT_HEADERFUNCTION, WriteMemoryCallback);
@@ -799,13 +848,22 @@ int http_post(const char* url, const char* proxy, int port, char* post_data, cha
         return 400;
     }
     
-    errornum = curl_easy_getinfo(c, CURLINFO_RESPONSE_CODE, &code );
-    msa_debug("CODE %ld", code);
-    msa_debug("HEADER\n  %s", chunk2.memory);    
+        errornum = curl_easy_getinfo(c, CURLINFO_RESPONSE_CODE, &code );
+        msa_debug("CODE %ld", code);
+
+        errornum = curl_easy_getinfo(c, CURLINFO_SIZE_UPLOAD, &size );
+        msa_debug("UPLOAD %f", size);
+
+        errornum = curl_easy_getinfo(c, CURLINFO_REQUEST_SIZE, &request_size );
+        msa_debug("REQUEST_SIZE %ld", request_size);
+
+	upload_size += size;
+	upload_size += request_size;
+
+    msa_debug("HEADER  %s\n header size = %d", chunk2.memory, chunk2.size);
     
-    if (header != NULL) {
-        *header = g_strdup(chunk2.memory);
-    }
+    download_size += chunk2.size;
+    download_size += chunk.size;
     
     if(chunk2.memory)
         g_free(chunk2.memory);
@@ -821,20 +879,21 @@ int http_post(const char* url, const char* proxy, int port, char* post_data, cha
     return code;
 }
 
-int http_upload_file(const char* url, const char* proxy, const int port, char** header, gchar** body, struct ProfileData *cp, struct UpFile *file)
+int http_upload_file(const char* url, const char* proxy, const int port, gchar** body, struct UpFile *file)
 {
     int i = 0;
     CURL *curl;
     CURLcode res;
     long code;
+    long request_size = 0;
+    double size = 0;
+
     struct MemoryStruct chunk,chunk2;
     struct curl_httppost *formpost=NULL;
     struct curl_httppost *lastptr=NULL;
     struct curl_slist *headerlist=NULL;
     static const char buf[] = "Expect:";
 
-    MSA_UNUSED(cp)
-
     chunk.memory = NULL;
     chunk.size = 0;
 
@@ -888,11 +947,21 @@ int http_upload_file(const char* url, const char* proxy, const int port, char**
 
         res = curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &code );
         msa_debug("CODE %ld", code);
-        msa_debug("HEADER\n  %s", chunk2.memory);        
+
+        res = curl_easy_getinfo(curl, CURLINFO_SIZE_UPLOAD, &size );
+        msa_debug("UPLOAD %f", size);
+
+        res = curl_easy_getinfo(curl, CURLINFO_REQUEST_SIZE, &request_size );
+        msa_debug("REQUEST_SIZE %ld", request_size);
+
+        upload_size += size;
+        upload_size += request_size;
+
+        msa_debug("HEADER  %s\n header size = %d", chunk2.memory, chunk2.size);
         
-        if (header != NULL) {
-            *header = g_strdup(chunk2.memory);
-        }
+        download_size += chunk2.size;
+        download_size += chunk.size;
+
         
         if(chunk2.memory)
             g_free(chunk2.memory);
