@@ -81,6 +81,9 @@ error_type msa_module_init(struct msa_module* d)
 static error_type driver_send(xmlDocPtr req, xmlDocPtr* response, const struct msa_module* info)
 {
     xmlDocPtr respXml = NULL;
+    xmlNodePtr respNode = NULL;
+    gchar *tmp;
+
     if (info == NULL) {
         msa_debug("msa_module in NULL! can't perform request!");
         return FAILURE;
@@ -94,10 +97,23 @@ static error_type driver_send(xmlDocPtr req, xmlDocPtr* response, const struct m
         g_warning("Error! Can't read request\n");
         return FAILURE;
     }
-    
+
+    upload_size = 0;
+    download_size = 0;
+
     request(req, &respXml, info);
     xmlFreeDoc(req);
 
+    respNode = xmlDocGetRootElement(respXml);
+    if(respNode != NULL){
+        tmp = g_strdup_printf("%d", upload_size);
+        xmlSetProp(respNode, BAD_CAST "upload", BAD_CAST tmp);
+        g_free(tmp);
+
+        tmp = g_strdup_printf("%d", download_size);
+        xmlSetProp(respNode, BAD_CAST "download", BAD_CAST tmp);
+        g_free(tmp);
+    }
     *response = xmlCopyDoc(respXml, 1);
     xmlFreeDoc(respXml);
     msa_debug("driver_send: END");
